/* LSDADecoder
Copyright (C) 2019  Pavel Cernohorsky

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.*/

#include "DWARFPrinter.h"

#include "GeneralPrinter.h"

#include <sstream>

namespace
{

void appendOrIfPrintedSetPrint(std::ostringstream& result, bool& printed)
{
  if (printed)
  {
    result << " | ";
  }
  printed = true;
}

}

//------------------------------------------------------------------------------

std::string DWARFPrinter::encodingToStr(DWARFEHEncoding encoding)
{
  std::ostringstream result;
  bool printed = false;
  
  if (encoding == DWARFEHEncodingVal::DW_EH_PE_omit)
  {
    result << "DW_EH_PE_omit";
    return result.str();
  }

  if ((encoding & DWARFEHEncodingVal::DW_EH_PE_absptr) == DWARFEHEncodingVal::DW_EH_PE_absptr)
  {
    appendOrIfPrintedSetPrint(result, printed);
    result << "DW_EH_PE_absptr";
  }
  if ((encoding & DWARFEHEncodingVal::DW_EH_PE_uleb128) == DWARFEHEncodingVal::DW_EH_PE_uleb128)
  {
    appendOrIfPrintedSetPrint(result, printed);
    result << "DW_EH_PE_uleb128";
  }
  if ((encoding & DWARFEHEncodingVal::DW_EH_PE_udata2) == DWARFEHEncodingVal::DW_EH_PE_udata2)
  {
    appendOrIfPrintedSetPrint(result, printed);
    result << "DW_EH_PE_udata2";
  }
  if ((encoding & DWARFEHEncodingVal::DW_EH_PE_udata4) == DWARFEHEncodingVal::DW_EH_PE_udata4)
  {
    appendOrIfPrintedSetPrint(result, printed);
    result << "DW_EH_PE_udata4";
  }
  if ((encoding & DWARFEHEncodingVal::DW_EH_PE_udata8) == DWARFEHEncodingVal::DW_EH_PE_udata8)
  {
    appendOrIfPrintedSetPrint(result, printed);
    result << "DW_EH_PE_udata8";
  }
  if ((encoding & DWARFEHEncodingVal::DW_EH_PE_sleb128) == DWARFEHEncodingVal::DW_EH_PE_sleb128)
  {
    appendOrIfPrintedSetPrint(result, printed);
    result << "DW_EH_PE_sleb128";
  }
  if ((encoding & DWARFEHEncodingVal::DW_EH_PE_sdata2) == DWARFEHEncodingVal::DW_EH_PE_sdata2)
  {
    appendOrIfPrintedSetPrint(result, printed);
    result << "DW_EH_PE_sdata2";
  }
  if ((encoding & DWARFEHEncodingVal::DW_EH_PE_sdata4) == DWARFEHEncodingVal::DW_EH_PE_sdata4)
  {
    appendOrIfPrintedSetPrint(result, printed);
    result << "DW_EH_PE_sdata4";
  }
  if ((encoding & DWARFEHEncodingVal::DW_EH_PE_sdata8) == DWARFEHEncodingVal::DW_EH_PE_sdata8)
  {
    appendOrIfPrintedSetPrint(result, printed);
    result << "DW_EH_PE_sdata8";
  }

  if ((encoding & DWARFEHEncodingVal::DW_EH_PE_pcrel) == DWARFEHEncodingVal::DW_EH_PE_pcrel)
  {
    appendOrIfPrintedSetPrint(result, printed);
    result << "DW_EH_PE_pcrel";
  }
  if ((encoding & DWARFEHEncodingVal::DW_EH_PE_textrel) == DWARFEHEncodingVal::DW_EH_PE_textrel)
  {
    appendOrIfPrintedSetPrint(result, printed);
    result << "DW_EH_PE_textrel";
  }
  if ((encoding & DWARFEHEncodingVal::DW_EH_PE_datarel) == DWARFEHEncodingVal::DW_EH_PE_datarel)
  {
    appendOrIfPrintedSetPrint(result, printed);
    result << "DW_EH_PE_datarel";
  }
  if ((encoding & DWARFEHEncodingVal::DW_EH_PE_funcrel) == DWARFEHEncodingVal::DW_EH_PE_funcrel)
  {
    appendOrIfPrintedSetPrint(result, printed);
    result << "DW_EH_PE_funcrel";
  }
  if ((encoding & DWARFEHEncodingVal::DW_EH_PE_aligned) == DWARFEHEncodingVal::DW_EH_PE_aligned)
  {
    appendOrIfPrintedSetPrint(result, printed);
    result << "DW_EH_PE_aligned";
  }

  return result.str();
}
