/* LSDADecoder
Copyright (C) 2019  Pavel Cernohorsky

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.*/

#include "LSDAPrinter.h"

#include "DWARFPrinter.h"
#include "GeneralPrinter.h"

#include <iostream>

namespace
{

//------------------------------------------------------------------------------

void printDWARFEHEncoding(const char* name, DWARFEHEncoding encoding)
{
  std::cout << name << " " << GeneralPrinter::printHex(encoding)
            << " (" << DWARFPrinter::encodingToStr(encoding) << ")\n";
}

//------------------------------------------------------------------------------

void printOptionalInt(const char* name, std::optional<int64_t> val)
{
  std::cout << name << " ";
  if (val)
  {
    std::cout << GeneralPrinter::printHex(*val);
  }
  else
  {
    std::cout << "null";
  }
  std::cout << '\n';
}

//------------------------------------------------------------------------------

void printHeader(const LSDAHeader& header)
{
  printDWARFEHEncoding("landingPadBaseEncoding", header.m_landingPadBaseEncoding);
  printOptionalInt("landingPadBase", header.m_landingPadBase);
  printDWARFEHEncoding("typeTableEntryEncoding", header.m_typeTableEntryEncoding);
  printOptionalInt("typeTableOffset", header.m_typeTableOffset);
  printOptionalInt("typeTableStart", header.m_typeTableStart);
  printDWARFEHEncoding("callSiteTableEntryEncoding", header.m_callSiteTableEntryEncoding);
  printOptionalInt("callSiteTableSize", header.m_callSiteTableSize);
  printOptionalInt("actionTableStart", header.m_actionTableStart);
}

//------------------------------------------------------------------------------

void printCallSiteTable(const std::vector<LSDACallSite>& callSiteTable)
{
  std::cout << "callSiteTableNumOfEntries " << callSiteTable.size() << '\n';
  for (size_t i = 0; i < callSiteTable.size(); ++i)
  {
    std::cout << "callSiteTableEntry" << i << '\n';
    printOptionalInt("  ipStart", callSiteTable[i].m_ipStart);
    printOptionalInt("  ipLength", callSiteTable[i].m_ipLength);
    printOptionalInt("  landingPad", callSiteTable[i].m_landingPad);
    printOptionalInt("  action", callSiteTable[i].m_action);
    std::cout << "  actionsNumOfEntries " << callSiteTable[i].m_actions.size() << '\n';
    for (size_t j = 0; j < callSiteTable[i].m_actions.size(); ++j)
    {
      std::cout << "  actionsEntry" << j << '\n';
      printOptionalInt("    address", callSiteTable[i].m_actions[j].m_address);
      std::cout << "    filter " << callSiteTable[i].m_actions[j].m_filter << '\n';
      std::cout << "    disp " << callSiteTable[i].m_actions[j].m_disp << '\n';
      if (callSiteTable[i].m_actions[j].m_type)
      {
        printOptionalInt("    typeOffset", callSiteTable[i].m_actions[j].m_type->m_typeOffset);
        printOptionalInt("    type", callSiteTable[i].m_actions[j].m_type->m_type);
        std::cout << "    typeIndirect " << callSiteTable[i].m_actions[j].m_type->m_typeIsIndirect
                  << '\n';
      }
      else
      {
        printOptionalInt("    type", std::nullopt);
      }
    }
  }
}

} // namespace

//------------------------------------------------------------------------------

void LSDAPrinter::print(const LSDA& lsda)
{
  printHeader(lsda.m_header);
  printCallSiteTable(lsda.m_callSiteTable);
  std::cout.flush();
}
